import { Array } from "@siteimprove/alfa-array";
import type { Device } from "@siteimprove/alfa-device";
import { Element } from "@siteimprove/alfa-dom";
import { Result } from "@siteimprove/alfa-result";
import { Context } from "@siteimprove/alfa-selector";
import { Set } from "@siteimprove/alfa-set";
import { Color } from "./color.js";
import { ColorErrors } from "./color-error.js";
/**
 * @public
 */
export declare class Layer {
    static of(colors: Iterable<Color.Resolved>, opacity: number): Layer;
    private readonly _colors;
    private readonly _opacity;
    protected constructor(colors: ReadonlyArray<Color.Resolved>, opacity: number);
    get colors(): ReadonlyArray<Color.Resolved>;
    get opacity(): number;
}
/**
 * @public
 */
export declare namespace Layer {
    /**
     * Gather all background layers of an element.
     * The layers can be set both by the element's own background property, and
     * by its ancestors, depending on transparency.
     * Since we do not have layout information, we simply assume that the DOM tree
     * ancestry mimics the actual layout of boxes.
     */
    function getLayers(element: Element, device: Device, context?: Context, opacity?: number, ignoredInterposedDescendants?: Set<Element>): Result<Array<Layer>, ColorErrors<"layer">>;
    /**
     * Merge colors in a layer with colors in an existing backdrop.
     */
    function merge(backdrops: Array<Color.Resolved>, layer: Layer): Array<Color.Resolved>;
}
//# sourceMappingURL=get-layers.d.ts.map